/*
 * uses global functions in config.js (passwordManager etc.)
 */

var credentials_ok = null;
if (window.arguments == 'newVersion') {
	credentials_ok = false;
}

function credChanged() { 
	// import button
	if (document.getElementById("host").value != '')
		document.getElementById("importCert").setAttribute('disabled', false);
	else
		document.getElementById("importCert").setAttribute('disabled', true);
	// credentials
	document.getElementById("checkCredentials").setAttribute('disabled', false);
	credentials_ok = false;
	return true;
}
	
function check_credentials() {
	var user = document.getElementById("user").value;
	var password = document.getElementById("password").value;
	var host = document.getElementById("host").value;
	var dir = document.getElementById("dir").value;
	var useSSL = document.getElementById("useSSL").checked;
	
	if (check_login(useSSL, host, dir, user, password) !== false) {
		credentials_ok = true;
		// anonymous usage tracker (if host == public domain-name, hash it)
		
		var isDomain = /(([a-z0-9]+)\.)*([a-z0-9\-]+)\.[a-z]+/g;
		if ( isDomain.test(host) ) {
			host = hash(host);
		}
		var userHost = hash(user+'@'+host);
		
		if (confirmDlg(strings.trackingQuestionTitle, 
		strings.trackingQuestionText + "\n\n" +
		'Host: ' + host + "\n" +
		'Account: ' + userHost + "\n" +
		'Version: ' +VERSION
		)) {
		
			var req = new XMLHttpRequest();
			req.open('POST', 'http://www.nohl.eu/tracker/tine-purse.php', false);
			var post = 'host='+host+'&userHost='+userHost+'&version='+VERSION;
			req.setRequestHeader("Content-type", 'application/x-www-form-urlencoded');
			req.setRequestHeader("Content-length", post.length);
			req.setRequestHeader("Connection", 'close');
			req.send(post);
			
			if (req.status == 200 && req.responseXML.getElementsByTagName("message")[0].innerHTML == 'success' ) {
				credentials_ok = true;
				document.getElementById("checkCredentials").setAttribute('disabled', true);
				save_credentials();
				return true;
			} 
		} else {
			credentials_ok = false;
		}
	} else {
		// prompt credentials wrong
		alertDlg(strings.credentialsFailedTitle, strings.credentialsFailedText + 
			tineUrl(
				document.getElementById("useSSL").checked, 
				document.getElementById("host").value,
				document.getElementById("dir").value
			) + '. ');
		credentials_ok = null;
	}
}

function save_credentials() {
	var user = document.getElementById("user").value;
	var password = document.getElementById("password").value;
	// remove existing
	if (passwordManager.countLogins('tinepurse://default', null, 'Tine 2.0 Login') > 0) {
		passwordManager.removeAllLogins();
	}
	// save new
	var loginInfo = new nsLoginInfo(
		'tinepurse://default', null, 'Tine 2.0 Login', user, password, '', ''
	);
	passwordManager.addLogin(loginInfo);
	// save prefs
	prefs.setCharPref('host', document.getElementById('host').value); 
	prefs.setCharPref('dir', document.getElementById('dir').value);
	prefs.setBoolPref('useSSL', document.getElementById('useSSL').checked);
	prefs.setCharPref('version', VERSION);
}
	
function hash(txt) {
	var ch = Components.classes["@mozilla.org/security/hash;1"]
		   .createInstance(Components.interfaces.nsICryptoHash);
	var converter =
		   Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
			 createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
	
	var result = {};
	converter.charset = "UTF-8";
	var data = converter.convertToByteArray(txt, result);
	
	ch.initWithString('SHA1');
	ch.update(data, data.length);
	
	var hash = ch.finish(false);
	
	function toHexString(charCode)
	{
	  return ("0" + charCode.toString(16)).slice(-2);
	}
	
	return [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");
}
	
function check_login(useSSL, host, dir, user, password) {
	var url = tineUrl(
		useSSL, 
		host,
		dir
	);

	resp = tineRequest(url, 
		[{
			'jsonrpc': '2.0', 
			'method': 'Tinebase.login', 
			'params': {
				'username': user, 
				'password': password
			}, 
			'id': 1
		}]
	); 
	
	// if login success logout again
	if (resp) {
		var jsonKey = resp.jsonKey; 
		var aname = resp.account.accountFullName; 
		if (tineRequest(url, [{
			'jsonrpc': '2.0', 
			'method': 'Tinebase.logout', 
			'params': {}, 
			'id': 1
		}], jsonKey) ){
			return aname;
		}
	}
	
	return false;
}
	
function make_default() {
	/* desktop-file-edit --set-key=Exec --set-value="`pwd tine20` %u" content/skin/os/tinepurse.desktop
	 *
	 */
}

/*
 * Dialog handling
 */

function onClose(ok) { 
	// respect Windows' cancel button
	if (document.getElementById('tinepurse_config').instantApply || ok) {
		
		// user changed credentials since last save
		if (credentials_ok === false) {
			check_credentials();
		}
		
	}
	
	jndump("Restart after configuring...\n");
	var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
		.getService(Components.interfaces.nsIAppStartup);
	if(confirmDlg(strings.configRestartTitle, strings.configRestartText))
		appStartup.quit(Components.interfaces.nsIAppStartup.eForceQuit | Components.interfaces.nsIAppStartup.eRestart);
	else
		appStartup.quit(Components.interfaces.nsIAppStartup.eForceQuit);
}

function onLoad() { 
	
	function fieldFromPref(pref) {
		var element = document.getElementById(pref); 
		if (element.tagName == 'checkbox') {
			element.checked = prefs.prefHasUserValue(pref)? prefs.getBoolPref(pref) : true;
		} else { 
			element.value = prefs.prefHasUserValue(pref)? prefs.getCharPref(pref) : '';
		}
	}
	
	function fieldsFromPwdMgr(user, password) {
		var logins = passwordManager.findLogins({}, 'tinepurse://default', null, 'Tine 2.0 Login');
		for (i in logins) {
			document.getElementById(user).value = logins[i].username;
			document.getElementById(password).value = logins[i].password;
		}
	}
	
	fieldFromPref('host');
	fieldFromPref('useSSL');
	fieldFromPref('dir');
	
	fieldsFromPwdMgr('user', 'password');
	
}

/*
 * Import user certs
 */

function import_certificate() { 
	var filePicker = Components.classes["@mozilla.org/filepicker;1"]
		.createInstance(Components.interfaces.nsIFilePicker);
	filePicker.init(window, strings.configCertImportTitle, Components.interfaces.nsIFilePicker.modeOpen);
	filePicker.appendFilter(strings.configCertImportFilter, '*.crt;*.pem');
	
	if (filePicker.show() == Components.interfaces.nsIFilePicker.returnOK) {
		// read choosen cert into var cert
		var io = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService);
		var channel = io.newChannelFromURI( io.newFileURI(filePicker.file) );
		var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
			.getService(Components.interfaces.nsIScriptableInputStream); 
		var input = channel.open();
		scriptableStream.init(input); 
		var cert = scriptableStream.read(input.available());
		scriptableStream.close(); 
		input.close(); 
		// process cert
		var certDB = Components.classes["@mozilla.org/security/x509certdb;1"]
			.getService(Components.interfaces.nsIX509CertDB2);
		var beginCert = "-----BEGIN CERTIFICATE-----";
		var endCert = "-----END CERTIFICATE-----";
		var begin = cert.indexOf(beginCert);
		var end = cert.indexOf(endCert);  
		cert = cert.substring(begin + beginCert.length, end);
		cert = cert.replace(/[\r\n]/g, "");
		certDB.addCertFromBase64(cert, ',,', "Tine 2.0 Purse import");
		// override errors
		var certDBEnum = certDB.getCerts().getEnumerator(); 
		var certNew = [], certNewC = 0, tmpCert = null; 
		while (certDBEnum.hasMoreElements()) { 
			tmpCert = certDBEnum.getNext().QueryInterface(Components.interfaces.nsIX509Cert);
			if (tmpCert.nickname.substr(0, 20) != 'Builtin Object Token' ) {
				// if any additional certs are found, ask later for their usage
				// certNew[certNewC].verifyForUsage(Components.interfaces.nsIX509Cert.SERVER_CERT) == 0 for Server?!
				jndump('Certificate found: ' + tmpCert.nickname + "\n");
				certNew[certNewC] = tmpCert;
				certNewC = certNewC + 1;
			}
		} 
		if (certNewC > 0) {
			var items = [];
			for(i in certNew) {
				items.push(certNew[i].nickname);
			}
			var certSelectC = selectDlg(strings.configCertChooseTitle, strings.configCertChooseText + "\n\n" + strings.configCertChooseTextHost + document.getElementById("host").value + '. ', items);
			var port = -1;
			var host = document.getElementById("host").value;
			if (document.getElementById("host").value.indexOf(':') !== -1) {
				port = document.getElementById("host").value.substr( document.getElementById("host").value.indexOf(':')+1 );
				host = document.getElementById("host").value.substr(0, document.getElementById("host").value.indexOf(':') );
			}
			// now add the chosen cert to the override list
			if (certSelectC != false) {
				jndump('Certificate selected ' + certNew[certSelectC.value].nickname + "\n");
				var certOverride = Components.classes["@mozilla.org/security/certoverride;1"]
					.getService(Components.interfaces.nsICertOverrideService);
				certOverride.rememberValidityOverride(
					host,
					port, 
					certNew[certSelectC.value],
					Components.interfaces.nsICertOverrideService.ERROR_UNTRUSTED | Components.interfaces.nsICertOverrideService.ERROR_MISMATCH | Components.interfaces.nsICertOverrideService.ERROR_TIME,
					false
				);
				jndump('Certificate ' + certNew[certSelectC.value].nickname + ' overrides host ' + document.getElementById("host").value + "\n");
			}
		}
	}
}
